@@ -24,6 +24,11 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <netdb.h>
+
+#include <gconf/gconf-client.h>
 
 #include <libgnome/libgnome.h>
 #include <libgnomeui/libgnomeui.h>
@@ -34,12 +39,17 @@
 #include "ice.h"
 #include "save.h"
 #include "command.h"
-#include "splash.h"
+#include "splash-widget.h"
+#include "util.h"
+#include "gsm-sound.h"
+#include "gsm-gsd.h"
 
 /* Flag indicating autosave - user won't be prompted on logout to 
  * save the session */
 gboolean autosave = FALSE;
 
+gboolean logout_prompt = TRUE;
+
 /* Flag indicating that an independant session save is taking place */
 gboolean session_save = FALSE;
 
@@ -55,7 +65,7 @@ gboolean failsafe = FALSE;
 guint purge_delay = 30000;
 
 /* Wait period for clients to save. */
-guint warn_delay = 10000;
+guint warn_delay = 30000;
 
 /* Wait period for clients to die during shutdown. */
 guint suicide_delay = 10000;
@@ -121,14 +131,121 @@ set_lang (void)
     }
 }
 
+static void
+update_boolean (GConfClient *client,
+		guint cnxn_id,
+		GConfEntry *entry,
+		gpointer user_data)
+{
+  GConfValue *value;
+  gboolean *bool_value = user_data;
+
+  value = gconf_entry_get_value (entry);
+  *bool_value = gconf_value_get_bool (value);
+
+  gsm_verbose ("%s is now %s.\n", 
+	       gconf_entry_get_key (entry), 
+	       *bool_value ? "On" : "Off");
+}
+
+/* returns the hostname */
+static gchar *
+get_hostname (gboolean readable)
+{
+  static gboolean init = FALSE;
+  static gchar *result = NULL;
+  
+  if (!init)
+    {
+      char hostname[256];
+      
+      if (gethostname (hostname, sizeof (hostname)) == 0)
+	result = g_strdup (hostname);
+		
+      init = TRUE;
+    }
+	
+  if (result)
+    return result;
+  else
+    return readable ? "(Unknown)" : NULL;
+}
+
+/* Check if a DNS lookup on `hostname` can be done */
+static gboolean
+check_for_dns (void)
+{
+  char *hostname;
+  
+  hostname = get_hostname (FALSE);
+	
+  if (!hostname)
+    return FALSE;
+
+  /*
+   * FIXME:
+   *  we should probably be a lot more robust here
+   */
+  if (!gethostbyname (hostname))
+    return FALSE;
+  
+  return TRUE;
+}
+
+enum {
+  RESPONSE_LOG_IN,
+  RESPONSE_TRY_AGAIN
+};
+    
+static void
+gnome_login_check (void)
+{
+  GtkWidget *tmp_msgbox = NULL;
+
+  while (!check_for_dns ())
+    {
+      if (!tmp_msgbox)
+	{
+	  tmp_msgbox = gtk_message_dialog_new (NULL, 0,
+					       GTK_MESSAGE_WARNING,
+					       GTK_BUTTONS_NONE,
+					       _("Could not look up internet address for %s.\n"
+						 "This will prevent GNOME from operating correctly.\n"
+						 "It may be possible to correct the problem by adding\n"
+						 "%s to the file /etc/hosts."),
+					       get_hostname(TRUE), get_hostname(TRUE));
+  
+	  gtk_dialog_add_buttons (GTK_DIALOG (tmp_msgbox),
+				  _("Log in Anyway"), RESPONSE_LOG_IN,
+				  _("Try Again"), RESPONSE_TRY_AGAIN,
+				  NULL);
+	  
+	  gtk_window_set_position (GTK_WINDOW (tmp_msgbox), GTK_WIN_POS_CENTER);
+	}
+
+      gtk_dialog_set_default_response (GTK_DIALOG (tmp_msgbox), RESPONSE_TRY_AGAIN);
+
+      if (RESPONSE_TRY_AGAIN != gtk_dialog_run (GTK_DIALOG (tmp_msgbox)))
+	{
+	  gtk_widget_destroy (tmp_msgbox);
+  	  break;
+	}
+    }
+}
 
 int
 main (int argc, char *argv[])
 {
-  gboolean splashing;
   char *ep;
   char *session_name_env;
   Session *the_session;
+  GConfClient *gconf_client;
+  gboolean splashing;
+  GError *err;
+  int status;
+  
+  if (getenv ("GSM_VERBOSE_DEBUG"))
+    gsm_set_verbose (TRUE);
   
   set_lang();
 
@@ -137,12 +254,31 @@ main (int argc, char *argv[])
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
-  /* We do this as a separate executable, and do it first so that we
-   * make sure we have a absolutely clean setup if the user blows
-   * their configs away with the Ctrl-Shift hotkey.
+  /* gnome_login_check() needs gtk initialized. however, these do
+   * sanity checks for gnome stuff, so we don't want to initialize
+   * gnome yet
    */
-  system ("gnome-login-check");
+  gtk_init (&argc, &argv);
+  gnome_login_check ();
 
+  err = NULL;
+  g_spawn_command_line_sync (GCONF_SANITY_CHECK, NULL, NULL, &status, &err);
+  if (err != NULL)
+    {
+      g_printerr ("Failed to run gconf-sanity-check-2: %s\n",
+                  err->message);
+      g_error_free (err);
+      /* who knows what's wrong, just continue */
+    }
+  else
+    {
+      if (WIFEXITED (status) && WEXITSTATUS (status) != 0)
+        {
+          g_printerr ("gconf-sanity-check-2 did not pass, logging back out\n");
+          exit (1);
+        }
+    }
+  
   gnome_program_init("gnome-session", VERSION, LIBGNOMEUI_MODULE, argc, argv, 
 		  GNOME_PARAM_POPT_TABLE, options,
 		  NULL);
@@ -168,11 +304,27 @@ main (int argc, char *argv[])
 
   ignore (SIGPIPE);
 
-  /* Read in config options */
+  /* Read in config options */  
+  gconf_client = gconf_client_get_default ();
+
+  gconf_client_add_dir (gconf_client, GSM_GCONF_CONFIG_PREFIX, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  splashing      = gconf_client_get_bool (gconf_client, SPLASH_SCREEN_KEY, NULL);
+  autosave       = gconf_client_get_bool (gconf_client, AUTOSAVE_MODE_KEY, NULL);
+  logout_prompt  = gconf_client_get_bool (gconf_client, LOGOUT_PROMPT_KEY, NULL);
+
+  gconf_client_notify_add (gconf_client,
+			   AUTOSAVE_MODE_KEY,
+			   update_boolean,
+			   &autosave, NULL, NULL);
+
+  gconf_client_notify_add (gconf_client,
+			   LOGOUT_PROMPT_KEY,
+			   update_boolean,
+			   &logout_prompt, NULL, NULL);
+
+  g_object_unref (gconf_client);
+
   gnome_config_push_prefix (GSM_OPTION_CONFIG_PREFIX);
-  splashing = gnome_config_get_bool (SPLASH_SCREEN_KEY "=" SPLASH_SCREEN_DEFAULT);
-  autosave = gnome_config_get_bool (AUTOSAVE_MODE_KEY "=" AUTOSAVE_MODE_DEFAULT);
- 
 
   /* If the session wasn't set on the command line, but we see a
    * GDM_GNOME_SESSION env var, use that.  The user is using the GDM chooser
@@ -186,6 +338,11 @@ main (int argc, char *argv[])
   if(session_name == NULL) {
 	/* If there is no key specified, fall back to the default session */	
 	session_name = gnome_config_get_string (CURRENT_SESSION_KEY "=" DEFAULT_SESSION);
+	/* if key was specified but is blank, just use the default */
+	if (!*session_name) {
+		g_free (session_name);
+		session_name = g_strdup (DEFAULT_SESSION);
+	}
   }
    
   gnome_config_pop_prefix ();
@@ -193,17 +350,23 @@ main (int argc, char *argv[])
   if(failsafe)
 	session_name = FAILSAFE_SESSION;
   
-  session_name_env = g_strconcat ("GNOME_DESKTOP_SESSION_ID=", g_strdup (session_name), NULL);
+  session_name_env = g_strconcat ("GNOME_DESKTOP_SESSION_ID=", session_name, NULL);
   putenv (session_name_env);
   the_session = read_session (session_name);
 
+  gsm_sound_login ();
+
+  gsm_gsd_start ();
+
   if (splashing)
-    start_splash (49.0);
+    splash_start ();
 
   start_session (the_session);
 
   gtk_main ();
 
+  gsm_sound_logout ();
+
   clean_ice ();
 
   /* If a logout command was set, the following will not return */
